/*
 * Gaim-Audacious plugin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#define GAIM_PLUGINS 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "gtkplugin.h"
#include "util.h"
#include "debug.h"
#include "connection.h"
#include "version.h"
#include <audacious/beepctrl.h>

#define GAIMAUD_PLUGIN_ID	"gaim_audacious"

#define OPT_GAIMAUD 		"/plugins/gaim_audacious"
#define OPT_PROCESS_STATUS	OPT_GAIMAUD "/process_status"
#define OPT_PROCESS_USERINFO	OPT_GAIMAUD "/process_userinfo"

#define SONG_TOKEN		"%song"

#define aud_debug(fmt, ...)	gaim_debug(GAIM_DEBUG_INFO, "Gaim-Audacious", \
					fmt, ## __VA_ARGS__);
#define aud_error(fmt, ...)	gaim_debug(GAIM_DEBUG_ERROR, "Gaim-Audacious", \
					fmt, ## __VA_ARGS__);

static gint timeout_tag = 0;


static void aud_process(gchar *aud_info);


static gchar *stored_status = NULL;
static gchar *stored_userinfo = NULL;
static gchar *pushed_userinfo = NULL;


static gboolean
watchdog_func(void)
{
	gint playpos;
	gchar *song, *buf;
	gint session = 0; // default session.
	gboolean rv = TRUE;

	if(!xmms_remote_is_playing(0)) { // audacious isn't running.
		aud_process(NULL);
		return rv;
	}

	playpos = xmms_remote_get_playlist_pos(session);
	song = xmms_remote_get_playlist_title(session, playpos);

	if(song) {
		buf = g_strdup_printf("%s", song);
		aud_process(buf);
		g_free(song); g_free(buf);
		song = NULL; buf = NULL;
	}
	else {
		aud_process(NULL);
	}

	return rv;
}

static void
aud_process_status(GaimConnection *gc, gchar *aud_info)
{
	gchar *new;
	const gchar *old, *proto;
	GaimAccount *account;
	GaimPresence *presence;
	GaimPlugin *prpl;
	GaimPluginProtocolInfo *prpl_info;
	GaimStatus *status;

	account = gaim_connection_get_account(gc);
	presence = gaim_account_get_presence(account);

	proto = gaim_account_get_protocol_id(account);
	prpl = gaim_find_prpl(proto);
	g_return_if_fail(prpl != NULL);

	prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(prpl);
	g_return_if_fail(prpl_info != NULL && prpl_info->set_status != NULL);

	status = gaim_presence_get_active_status(presence);
	g_return_if_fail(status != NULL);

	old = gaim_status_get_attr_string(status, "message");
//	g_return_if_fail(old != NULL && strlen(old) != 0); // emit log message too frequently.
	if(old == NULL || strlen(old) == 0)
	   return;

	// if old contains token, compare to stored_status message.
	if(strstr(old, SONG_TOKEN)){
		if(stored_status && strcmp(stored_status, old)){ //different
			g_free(stored_status);
			stored_status = NULL;
		}
		if (!stored_status) {
			stored_status = g_strdup(old);
		}
	}

	/* construct our new status message */
	if(aud_info){
		new = gaim_strreplace(stored_status, SONG_TOKEN, aud_info);
	}
	else {
		new = g_strdup("No song being played.");
	}

	g_return_if_fail(new != NULL);

	/* only set the status message if the text has changed */
	if (g_ascii_strcasecmp(old, new) != 0) {
		gaim_status_set_attr_string(status, "message", new);
		prpl_info->set_status(account, status);
	}

	g_free(new);
}


static void
aud_process_userinfo(GaimConnection *gc, gchar *aud_info)
{
	gchar *new;
	const gchar *old, *proto;
	GaimAccount *account;
	GaimPlugin *prpl;
	GaimPluginProtocolInfo *prpl_info;

	account = gaim_connection_get_account(gc);

	proto = gaim_account_get_protocol_id(account);
	prpl = gaim_find_prpl(proto);
	g_return_if_fail(prpl != NULL);

	prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(prpl);
	g_return_if_fail(prpl_info != NULL && prpl_info->set_status != NULL);

	/* retrieve the old user info */
	old = gaim_account_get_user_info(account); // it's always from account.xml!
//	g_return_if_fail(old != NULL && strlen(old) != 0); //emit log message too frequently.
	if(old == NULL || strlen(old) == 0)
		return;

	// if old contains token, compare to stored_status message.
	if(strstr(old, SONG_TOKEN)){
		if(stored_userinfo && strcmp(stored_userinfo, old)){ //different
			g_free(stored_userinfo);
			stored_userinfo = NULL;
		}
		if (!stored_userinfo) {
			stored_userinfo = g_strdup(old);
		}
	}

	/* construct our new status message */
	if(aud_info){
		new = gaim_strreplace(stored_userinfo, SONG_TOKEN, aud_info);
	}
	else {
		new = g_strdup("No song being played.");
	}

	g_return_if_fail(new != NULL);

	/* only set the user info if the text has changed */
	if (!pushed_userinfo || g_ascii_strcasecmp(pushed_userinfo, new) != 0) {
		g_free(pushed_userinfo);
		pushed_userinfo = g_strdup(new);
		prpl_info->set_info(gc, new);
	}

	g_free(new);
}

static void
aud_process(gchar *aud_info)
{
	GList *l;
	GaimConnection *gc;

	for (l = gaim_connections_get_all(); l != NULL; l = l->next) {
		gc = (GaimConnection *) l->data;

		/* make sure we're connected */
		if (gaim_connection_get_state(gc) != GAIM_CONNECTED) {
			continue;
		}

		if (gaim_prefs_get_bool(OPT_PROCESS_USERINFO)) {
			aud_process_userinfo(gc, aud_info);
		}

		if (gaim_prefs_get_bool(OPT_PROCESS_STATUS)) {
			aud_process_status(gc, aud_info);
		}
	}
}


static gboolean
load_plugin(GaimPlugin *plugin)
{

	gchar *remote_uri;

	remote_uri = getenv("AUDTOOL_REMOTE_URI");
	audacious_set_session_uri(remote_uri);

	timeout_tag = g_timeout_add(15*1000, (gpointer)watchdog_func, NULL);

	return TRUE;
}

static gboolean
unload_plugin(GaimPlugin *plugin)
{
	g_free(stored_status);
	stored_status = NULL;
	g_free(stored_userinfo);
	stored_status = NULL;
	g_free(pushed_userinfo);
	stored_status = NULL;

	return TRUE;
}

static GaimPluginPrefFrame *
get_plugin_pref_frame(GaimPlugin *plugin)
{
	GaimPluginPref *pref;
	GaimPluginPrefFrame *frame = gaim_plugin_pref_frame_new();

	/* create gtk elements for the plugin preferences */
	pref = gaim_plugin_pref_new_with_label("Gaim-Audacious Configuration");
	gaim_plugin_pref_frame_add(frame, pref);
	
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_PROCESS_STATUS,
			"Expand " SONG_TOKEN " to song info in the status message");
	gaim_plugin_pref_frame_add(frame, pref);
	
	pref = gaim_plugin_pref_new_with_name_and_label(OPT_PROCESS_USERINFO,
			"Expand " SONG_TOKEN " to song info in the user info");
	gaim_plugin_pref_frame_add(frame, pref);

	return frame;
}

static GaimPluginUiInfo pref_info =
{
	get_plugin_pref_frame
};

static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,				/**< type	*/
	GAIM_GTK_PLUGIN_TYPE,				/**< ui_req	*/
	0,						/**< flags	*/
	NULL,						/**< deps	*/
	GAIM_PRIORITY_DEFAULT,				/**< priority	*/
	GAIMAUD_PLUGIN_ID,				/**< id		*/
	"Gaim-Audacious",				/**< name	*/
	"1.0.0",					/**< version	*/
							/**  summary	*/
	"Automatically updates your Gaim status info	with the currently "
			"playing music in Audacious.",
							/**  desc	*/
	"Automatically updates your Gaim status info with the currently "
			"playing music in Audacious.",
	"Yoshiki Yazawa (yaz@honeyplanet.jp)",		/**< author	*/
	"http://www.honeyplanet.jp",			/**< homepage	*/
	load_plugin,					/**< load	*/
	unload_plugin,					/**< unload	*/
	NULL,						/**< destroy	*/
	NULL,						/**< ui_info	*/
	NULL,						/**< extra_info	*/
	&pref_info,					/**< pref info	*/
	NULL
};

static void
init_plugin(GaimPlugin *plugin)
{
	g_type_init();

	/* add plugin preferences */
	gaim_prefs_add_none(OPT_GAIMAUD);
	gaim_prefs_add_bool(OPT_PROCESS_STATUS, TRUE);
	gaim_prefs_add_bool(OPT_PROCESS_USERINFO, TRUE);
}

GAIM_INIT_PLUGIN(gaim_audacious, init_plugin, info)
